<?php

require_once(__DIR__ . '/path.php');

// $writes = [];
class FileStore
{
	private static function mstime($round = true)
	{
		if ($round) return round(microtime(true) * 1000);
		return microtime(true) * 1000;
	}

	function __construct($basePath, ...$paths)
	{
		$this->groupBy(Path::join($basePath, ...$paths));
	}
	static function writeRecursive($path, $data = null)
	{
		// TODO skip from upload_physical_path.
		$segments = explode(DIRECTORY_SEPARATOR, $path);
		$size = sizeof($segments);

		$currentPath = '';
		$isFile = $data !== null;

		foreach ($segments as $i => $part) {
			$currentPath .= DIRECTORY_SEPARATOR . $part;

			if ($isFile && ($i == $size - 1)) {
				file_put_contents($currentPath, $data);

				break;
			}
			if (!is_dir($currentPath)) {
				@mkdir($currentPath, 0777);
			}
		}
	}
	static function removeRecursive($path)
	{
		if (!is_dir($path) || is_link($path)) {
			@unlink($path);

			return;
		}

		$objects = scandir($path);
		foreach ($objects as $object) {
			if ($object == "." || $object == "..")
				continue;

			self::removeRecursive($path . DIRECTORY_SEPARATOR . $object);
		}
		@rmdir($path);
	}

	public function moveInto(string $fromPath, FileStore $targetStore, string $toPath = '') {
		$sourcePath = $this->getAbsoluteLocalUploadPath($fromPath);
		$targetPath = $targetStore->getAbsoluteLocalUploadPath($toPath);
		$this->resolveFilePath($sourcePath, $targetPath);
		
		self::writeRecursive(Path::directory($targetPath));
		return rename($sourcePath, $targetPath);
	}
	function upload(string $url, string $filename = null)
	{
		$path = $this->uploadDataURL($url, $filename);
		if (!$path) {
			$path = $this->uploadURL($url, $filename);
		}
		return $path;
	}

	static function dataURLExtension($dataurl)
	{
		$isDataURL = preg_match('/^data:[-\w]+\/(?:([\-\w\.]+)(?:\+[\-\w\.]+)?)/', $dataurl, $matches);
		if (!$isDataURL) {
			return false;
		}
		return $matches[1];
	}
	function uploadFile($file, $filename = null)
	{
		$tmp_name = $file["tmp_name"];
		$extension = Path::extension($file["name"]);

		list($fileAbsolutePath, $basename) = $this->resolveFilePath($extension, $filename);
		self::writeRecursive(Path::directory($fileAbsolutePath));

		move_uploaded_file($tmp_name, $fileAbsolutePath);
		return $basename;
	}
	function uploadDataURL(string $dataurl, string $filename = null)
	{
		$isDataURL = preg_match('/^data:[-\w]+\/(?:([\-\w\.]+)(?:\+[\-\w\.]+)?);base64,(.+)$/', $dataurl, $matches);

		if (!$isDataURL)
			return false;

		$extension = $matches[1];
		$base64 = $matches[2];

		return $this->uploadBase64($base64, $extension, $filename);
	}
	function uploadBase64(string $base64, string $extension, string $filename = null)
	{
		$data = base64_decode($base64);

		return $this->uploadContent($data, $extension, $filename);
	}
	function uploadContent(string $data, string $extension, string $filename = null): string
	{
		list($fileAbsolutePath, $basename) = $this->resolveFilePath($extension, $filename);

		self::writeRecursive($fileAbsolutePath, $data);
		return $basename;
	}
	function copyUpload($path)
	{

	}

	static function isFromUpload($url)
	{
		return substr($url, 0, strlen(upload_url_path)) == upload_url_path;
	}
	static function urlToLocal($url)
	{
		if (!self::isFromUpload($url)) {
			return false;
		}
		return substr($url, strlen(upload_url_path));
	}
	function uploadURL(string $url, string $filename = null)
	{
		$extension = Path::extension($url);
		if (!$extension) {
			return false;
		}

		list($fileAbsolutePath, $basename) = $this->resolveFilePath($extension, $filename);

		$localPath = self::urlToLocal($url);
		if ($localPath) {
			if (Path::equals($localPath, $this->getLocalUploadPath($basename))) {
				return $basename;
			}
		}
		// TODO MAKE OPTION THAT MAKES IT DISABLE FETCHING FROM OTHER FILES ON THE UPLOAD FOLDER

		$ch = curl_init($url);
		self::writeRecursive(Path::directory($fileAbsolutePath));

		$fp = fopen($fileAbsolutePath, 'wb');
		if (!$fp) {
			// reject("Failed $fileAbsolutePath " . self::pathGoUpLevels($fileAbsolutePath, 1));
		}
		curl_setopt($ch, CURLOPT_FILE, $fp);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_exec($ch);

		$http_code = curl_getinfo($ch, CURLINFO_RESPONSE_CODE);
		if ($http_code == 404) {
			@FileStore::doLog(@[
				"url" => @$url,
				"scanned" => Path::directory(Path::join(upload_physical_path, $localPath)),
				"scandir" => @scandir(Path::directory(Path::join(upload_physical_path, $localPath))),
				"filename" => @$filename,
				"extension" => @$extension,
				"fileAbsolutePath" => @$fileAbsolutePath,
				"basename" => @$basename
			]);
		}
		curl_close($ch);
		fclose($fp);

		return $basename;
	}

	private array $groups = [];
	function groupBy($group)
	{
		$this->groups[] = strval($group);
	}
	function popGroup()
	{
		array_pop($this->groups);
	}
	function getLastGroups($delta = 1)
	{
		return Path::join(...array_slice($this->groups, count($this->groups) - $delta));
	}
	public function getGroupChildren()
	{
		$path = $this->getAbsoluteDirectory();
		if (!$this->fileExists('./'))
			return [];

		return scandir($path);
	}
	private function getGroup()
	{
		return Path::join(...$this->groups);
	}
	function getDirectory()
	{
		return $this->getGroup();
	}
	function getAbsoluteDirectory()
	{
		return Path::absoluteToUpload($this->getDirectory());
	}
	function getLocalUploadPath(string $basename)
	{
		return Path::join($this->getDirectory(), $basename);
	}
	function getURLUploadPath(string $basename)
	{
		return Path::joinURL(upload_url_path, $this->getLocalUploadPath($basename));
	}
	function getAbsoluteLocalUploadPath(string $basename)
	{
		return Path::absoluteToUpload($this->getLocalUploadPath($basename));
	}

	public function resolveFilePath(string $extension, $filename)
	{
		if ($filename == null) {
			return $this->createUniquePath($extension);
		}

		$basename = $filename . ($extension == '' ? '' : '.' . $extension);
		$fileAbsolutePath = $this->getAbsoluteLocalUploadPath($basename);

		return [$fileAbsolutePath, $basename];
	}
	public function createUniquePath(string $extension = '')
	{
		$formAbsolutePath = self::getAbsoluteDirectory();

		while (true) {
			$name = strval(self::mstime()) . "_" . rand(0, 100_000);

			$basename = "$name";
			if ($extension) {
				$basename .= ".$extension";
			}
			$fileAbsolutePath = "$formAbsolutePath/$basename";

			if (!file_exists($fileAbsolutePath)) {
				break;
			}
		}

		return [$fileAbsolutePath, $basename];
	}


	public function readFile($basename)
	{
		$path = $this->getAbsoluteLocalUploadPath($basename);

		return file_get_contents($path);
	}
	public function fileExists($basename)
	{
		$path = $this->getAbsoluteLocalUploadPath($basename);

		return file_exists($path);
	}

	public function clearCurrentGroup()
	{
		return $this->removeFile('./');
	}
	public function removeFile($basename)
	{
		$path = $this->getAbsoluteLocalUploadPath($basename);

		self::removeRecursive($path);
	}


	public static function doLog($data)
	{
		$store = new FileStore("mockup_template_debug_logs");

		$store->uploadContent(json_encode($data, JSON_PRETTY_PRINT), "log", date("l jS \of F Y h-i-s A"));
	}

}


// "{".charCodeAt(0).toString(16); -> 0x7b
// "}".charCodeAt(0).toString(16); -> 0x7d
// print_r(eval("if (true) \x7b print_r(scandir('.'));\x7d;return 10;"));
